// 
keypad_sz = [70, 90];
keypad_hole_pos = [62, 82];

feather_sz = [50.8, 22.86];
feather_hole_pos = [45.72, 17.78];

rail_pos_y = keypad_sz[1] + 4;
display_rail_th = 5;
display_sz = [63.5, 55.888888888];
display_hole_pos = [58.42, 50.8];
display_angle = 30;
display_h0 = 8;

$fn = 36;

rail_sz_y = display_sz[1] * cos(display_angle);
ovl_y = keypad_sz[1] + 6 + rail_sz_y + 6 + feather_sz[1];
ovl_x = max(keypad_sz[0], display_hole_pos[0] + display_rail_th);

module keypad_holes() {
    for(x=[-.5, .5])
    for(y=[-.5, .5])
    translate([keypad_hole_pos[0] * x, keypad_hole_pos[1] * y])
        let($IDX = x + y*2 + 1.5) children();
}

module keypad_2d(holes=true) {
    difference() {
        square(keypad_sz, center=true);
        
        if(holes)
            keypad_holes() circle(d=2.7);
    }
}

module feather_holes() {
    for(x=[-.5, .5])
    for(y=[-.5, .5])
    translate([feather_hole_pos[0] * x, feather_hole_pos[1] * y])
        children();
}

module feather_2d(holes=true) {
    difference() {
        square(feather_sz, center=true);
        
        if(holes)
            feather_holes() circle(d=2.7);
    }
}

echo(rail_pos_y);
echo(rail_sz_y);
echo(keypad_sz[1]);

module base() {
    translate([0,0,-2.95])
    linear_extrude(height=3)
    difference() {
        translate([0, ovl_y / 2])
        square([ovl_x, ovl_y], center=true);

        translate([0, keypad_sz[1]/2])
        {
            // As the perfboard holes are a bit wonky, provide slots
            keypad_holes() {
                echo($IDX);
                rotate($IDX == 1 || $IDX == 2 ? 90 : 0)
                hull() {
                    translate([-1,0])
                    circle(d=2.7);
                    translate([1,0])
                    circle(d=2.7);
                }
            }
            square([keypad_hole_pos[0]-12, keypad_sz[1]-12], center=true);
            square([keypad_sz[0]-12, keypad_hole_pos[1]-12], center=true);
        }

        translate([0, keypad_sz[1] / 2 + ovl_y / 2])
        // rotate(90)
        {
            square([feather_hole_pos[0]-6, ovl_y - keypad_sz[1] - 8], center=true);
        }
        translate([0, ovl_y - feather_sz[1]/2 - 0]) {
            square([feather_sz[0]-6, feather_hole_pos[1]-6], center=true);
            feather_holes() circle(d=2.7);
        }
    }
}

module display_rail_2d() {
    c = (display_sz[1] + 5) * cos(display_angle);
    s = (display_sz[1] + 5) * sin(display_angle);
    h0 = display_h0;
    difference() {
        polygon([
            [0, 0],
            [0, h0 + s],
            [c, h0],
            [c, 0]
        ]);
        if(0)
        translate([c/2,0])
        square([20, 100], center=true);
    }
}

module display_rail_3d() {
    c = (display_sz[1] + 5) * cos(display_angle);
    s = (display_sz[1] + 5) * sin(display_angle);
    h0 = display_h0;

    rotate(-90)
    translate([-c,0,0])
    rotate([90,0,0]) {
        difference() {
            linear_extrude(height=5, center=true) display_rail_2d();

            translate([c, h0])
            rotate(-display_angle)
            translate([-display_sz[1] / 2 - 1, 0])
            {
                for(y = [-display_hole_pos[1]/2, display_hole_pos[1]/2]) {
                    translate([y, 0]) {
                        rotate([90,0,0]) {
                            cylinder(d=2.7, h=12, center=true);
                            translate([0,0,2.5])
                            cube([5.2,8,2.7], center=true);
                        }
                    }
                }
            }
        }
    }
}


base();

translate([-display_hole_pos[0]/2, keypad_sz[1] + 8, 0])
display_rail_3d();
translate([display_hole_pos[0]/2, keypad_sz[1] + 8, 0])
display_rail_3d();
